# Copyright 2020 Alexander Polishchuk
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
class BaseError(Exception):
    """Base connector error."""


class NodeError(BaseError):
    """Node returns an error."""


class NodeInvalidResponceError(NodeError):
    """Node returns a invalid response."""


class NodeUnsupportedError(NodeError, ValueError):
    """Node unsupported error."""

class NodeTooSmallTransactionAmount(NodeError):
    """Insufficient transaction amount for substract fee. """


class CurrencyUnsupportedError(NodeError, ValueError):
    """Currency unsupported error."""


class NetworkError(BaseError):
    """Node network error."""


class NetworkTimeoutError(NetworkError):
    """Node request riches timeout."""
