# Copyright 2020 Alexander Polishchuk
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from obm.connectors.base import DEFAULT_TIMEOUT, Connector
from obm.connectors.bitcoin import BitcoinCoreConnector
from obm.connectors.ethereum import GethConnector

__all__ = [
    "BitcoinCoreConnector",
    "GethConnector",
    "DEFAULT_TIMEOUT",
]

MAPPING = {
    "bitcoin-core": BitcoinCoreConnector,
    "geth": GethConnector,
}
SUPPORTED_CURRENCIES = list({node.currency for node in MAPPING.values()})
